%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Replicates matching function estimation algorithm from
%   "Beyond Cobb-Douglas: Flexibly Estimating Matching Functions with
%       Unobserved Matching Efficiency"
%
%   code written by Fabian Lange and Theodore Papageorgiou
%
%   3 Files created for use in Stata to generate figures on lines: 183,
%   303, 508
%      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear
format long g
%
load masterdata.mat

%%% Definition of parameters and variables used below
% lambda and theta ==   scaling factors used in program to go through support
%                       of arguments of the matching function (see step 2
%                       and 3 of Section 3 identification argument.) 
%                       lambda is used to go through the support of S
%                       theta is used to go through the support of A
% S == observed search
% A == matching efficiency / unobserved search effort

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Technical parameters            %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
supportsize = 100;  % size of the support for theta
hh = 0.1;           % the standand deviation of the kernel used to compute the conditional density is hh*(max-min) of the variable you're targeting, i.e. hh is a percentage
maxA = 300;         % maximum matching efficiency in grid (normalization at median vacancies is 100)
minA = 10;          % minimum matching efficiency in grid (normalization at median vacancies is 100)

% set randomness seed
rng('default')
rng(900)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Generate Vectors containing basic variables        %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

m = mJolts;              % We measure new matches and vacancies from JOLTS
s_u = s_u_new;           % s_u_new is the updated MPS index of unobserved search of unemployed updated to Dec 2017.
v = vJolts .* dfhindex;  % Recruiting = JOLTS*Davis index           

% compute stocks and UE rate for all unemployed workers (not by gender)
U  = Umale+Ufemale;
UE  = UEmale.*(Umale./(Umale+Ufemale))+UEfemale.*(Ufemale./(Umale+Ufemale));

% Generate NaNindex for any observation that has missing values.
NaNindex = isnan(v)+isnan(m)+isnan(s_u)+isnan(U)+isnan(UE)+isnan(NEmale)+...
    isnan(NEfemale)+isnan(EEmale)+isnan(EEfemale)+isnan(NEmaleY)+ ...
    isnan(NEfemaleY)+isnan(EEmaleY)+isnan(EEfemaleY)+isnan(UEmaleY)+...
    isnan(UEfemaleY)+isnan(Nmale)+isnan(Nfemale)+isnan(Emale)+isnan(Efemale)...
    +isnan(NmaleY)+isnan(NfemaleY)+isnan(EmaleY)+isnan(EfemaleY)+isnan(UmaleY)...
    +isnan(UfemaleY);
NaNindex = (NaNindex>0);
NaNindex = double(NaNindex);

% Pull out the part of the data that is not missing
v = v(~(NaNindex)==1);
m = m(~(NaNindex)==1);
s_u = s_u(~(NaNindex)==1);
vJolts = vJolts(~(NaNindex)==1);
dfhindex = dfhindex(~(NaNindex)==1);


UE = UE(~(NaNindex)==1);         % 
UEmale = UEmale(~(NaNindex)==1);
UEfemale = UEfemale(~(NaNindex)==1);
NEmale = NEmale(~(NaNindex)==1);
NEfemale = NEfemale(~(NaNindex)==1);
EEmale = EEmale(~(NaNindex)==1);
EEfemale = EEfemale(~(NaNindex)==1);
NEmaleY = NEmaleY(~(NaNindex)==1);
NEfemaleY = NEfemaleY(~(NaNindex)==1);
EEmaleY = EEmaleY(~(NaNindex)==1);
EEfemaleY = EEfemaleY(~(NaNindex)==1);
UEmaleY = UEmaleY(~(NaNindex)==1);
UEfemaleY = UEfemaleY(~(NaNindex)==1);
Nmale = Nmale(~(NaNindex)==1);
Nfemale = Nfemale(~(NaNindex)==1);
Emale = Emale(~(NaNindex)==1);
Efemale = Efemale(~(NaNindex)==1);
NmaleY = NmaleY(~(NaNindex)==1);
NfemaleY = NfemaleY(~(NaNindex)==1);
EmaleY = EmaleY(~(NaNindex)==1);
EfemaleY = EfemaleY(~(NaNindex)==1);
UmaleY = UmaleY(~(NaNindex)==1);
UfemaleY = UfemaleY(~(NaNindex)==1);
U = U(~(NaNindex)==1);
Umale = Umale(~(NaNindex)==1);
Ufemale = Ufemale(~(NaNindex)==1);
%
month2 = month; % hold on to original month and year, will use later
year2   = year;
%
month = month(~(NaNindex)==1);
year = year(~(NaNindex)==1);

% Deseasonalize variables. 
month_dum = dummyvar(month);
% \ = regression operator
b = month_dum\m;
 m = m-month_dum*b+mean(b);
b = month_dum\v;
 v = v-month_dum*b+mean(b);
b = month_dum\s_u;
 s_u=s_u-month_dum*b+mean(b);
 
% flows 
b = month_dum\UE;
UE=UE-month_dum*b+mean(b);
b = month_dum\NEmale;
NEmale=NEmale-month_dum*b+mean(b);
b = month_dum\NEfemale;
NEfemale =NEfemale-month_dum*b+mean(b);
b = month_dum\EEmale;
 EEmale=EEmale-month_dum*b+mean(b);
b = month_dum\EEfemale;
 EEfemale=EEfemale-month_dum*b+mean(b);
b = month_dum\NEmaleY;
 NEmaleY=NEmaleY-month_dum*b+mean(b);
b = month_dum\NEfemaleY;
 NEfemaleY=NEfemaleY-month_dum*b+mean(b);
b = month_dum\EEmaleY;
 EEmaleY=EEmaleY-month_dum*b+mean(b);
b = month_dum\EEfemaleY;
 EEfemaleY=EEfemaleY-month_dum*b+mean(b);
b = month_dum\UEmaleY;
 UEmaleY=UEmaleY-month_dum*b+mean(b);
b = month_dum\UEfemaleY;
 UEfemaleY=UEfemaleY-month_dum*b+mean(b);
% stocks
b = month_dum\Emale;
Emale=Emale-month_dum*b+mean(b);
b = month_dum\EmaleY;
EmaleY=EmaleY-month_dum*b+mean(b);
b = month_dum\Efemale;
Efemale=Efemale-month_dum*b+mean(b);
b = month_dum\EfemaleY;
EfemaleY=EfemaleY-month_dum*b+mean(b);
b = month_dum\Nmale;
Nmale=Nmale-month_dum*b+mean(b);
b = month_dum\NmaleY;
NmaleY=NmaleY-month_dum*b+mean(b);
b = month_dum\Nfemale;
Nfemale=Nfemale-month_dum*b+mean(b);
b = month_dum\NfemaleY;
NfemaleY=NfemaleY-month_dum*b+mean(b);
b = month_dum\U;
U=U-month_dum*b+mean(b);
b = month_dum\UmaleY;
UmaleY=UmaleY-month_dum*b+mean(b);
b = month_dum\UfemaleY;
UfemaleY=UfemaleY-month_dum*b+mean(b);
%      
%
% compute search effort for other groups
s_nm = (s_u).*(NEmale./UE);
s_nf = (s_u).*(NEfemale./UE);
s_em = (s_u).*(EEmale./UE);
s_ef = (s_u).*(EEfemale./UE);
s_nmY = (s_u).*(NEmaleY./UE);
s_nfY = (s_u).*(NEfemaleY./UE);
s_emY = (s_u).*(EEmaleY./UE);
s_efY = (s_u).*(EEfemaleY./UE);
s_umY = (s_u).*(UEmaleY./UE);
s_ufY = (s_u).*(UEfemaleY./UE);
%
% construct total observed search effort measure accounting for
% heterogeneity and MPS 
S = s_u.*U+Nmale.*s_nm+Nfemale.*s_nf+Emale.*s_em+Efemale.*s_ef+...
    NmaleY.*s_nmY+NfemaleY.*s_nfY+EmaleY.*s_emY+EfemaleY.*s_efY+...
    UmaleY.*s_umY+UfemaleY.*s_ufY;

% Write output to be used in Figures 1 & 2 @ into .csv-file to be read by Stata
Fig_main1 = [v vJolts dfhindex month year s_u EEfemale EEfemaleY EEmale EEmaleY NEfemale NEfemaleY NEmale ...
    NEmaleY UEfemale UEfemaleY UEmale UEmaleY Efemale EfemaleY Emale EmaleY Nfemale NfemaleY Nmale ...
    NmaleY Ufemale UfemaleY Umale UmaleY S];
csvwrite('figmain_1.csv',Fig_main1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Start Estimating Algorithm      %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% NORMALIZATION
%
vstar = prctile(v,50); % normalize A at median of v
[~, point_vstar] = min(abs(v-vstar));
mstar = m(point_vstar);
Sstar =  S(point_vstar);
%
% value of matching efficiency at normalization point
Astartrue = 100;
%
% For the handles (theta, lambda), we define the points in support of
% S and A that they can vary over 
supportlambda = S./Sstar; % use the values of S only at the observed points
supporttheta = linspace(minA/Astartrue,maxA/Astartrue,supportsize)'; % this determines the min and max A
supportA = Astartrue*supporttheta;
%
%
SVECTOR = supportlambda*Sstar;
%
% for kernel
hsvv = hh*range(v(:));
hsu = hh*max([range(S(:)) 1]);
%
%
% FAS1 = Matrix F(A|S) = contains the conditional distribution of matching efficiency A across S 
% each column is a distribution conditional on the point in S defined by a value of S 
% Described in Section 6.2 of paper.
FAS1 = zeros(supportsize,size(supportlambda,1)); % Initiate F(A|S) as an zero-matrix
for il = 1:size(supportlambda,1) % loop over the handle lambda over the S values
    lambda = supportlambda(il);
    for it = 1:supportsize % loop over handle theta corresponding to A
        theta    = supporttheta(it);
        y = 1*(m(:)<=theta*lambda*mstar);  % This does both Step 2 and 3 in Section 3. 
                                           % y is a vector that indicates all observations 
                                           % for which the observed number of hires is 
                                           % below that implied by a specific match efficiency
                                           % theta and a specific value of S.
                                           
        % find G(m|s(i),v(j)) -- for each pair (s',v') we want to estimate the distribution over matches
        % below some value m. To do this, we average observed proportions
        % of m below this value across values of (s,v) that are close to
        % (s',v') -- weighted by a kernel to put more weight on close
        % values.                                           
        FAS1(it, il) = ...
            sum(y.*mvnpdf([((v(:)-theta*lambda*vstar)/hsvv) ((S(:)-lambda*Sstar)/hsu)]))/sum(mvnpdf([((v(:)-theta*lambda*vstar)/hsvv) ((S(:)-lambda*Sstar)/hsu)]));
    end
end

% Predict efficiency A at each (M,S,V) 
% (since (S,V,A) map one-to-one into M and we can just invert it)
AIMPLIED = zeros(size(m,1),1); % aggregate matching efficiency
for i=1:size(m,1)
     y = 1*(m(:)<=m(i));
     Gmsv = sum(y.*mvnpdf([((v(:)-v(i))/hsvv) ((S(:)-S(i))/hsu)]))/sum(mvnpdf([((v(:)-v(i))/hsvv) ((S(:)-S(i))/hsu)])); % find G_m|s(i),v(i) at point m(i)
    %
    [~, POINTS] = min(abs(SVECTOR-S(i)));
    [~, POINTA] = min(abs(FAS1(:,POINTS)-Gmsv));
    AIMPLIED(i,1) = supportA(POINTA);
end
%
A = AIMPLIED; % aggregate matching efficiency
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Results                           %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% PREDICT M FOR GIVEN VALUES OF A, V, S 
% this is basically the matching function
%
mimplied = zeros(size(m,1),1);
for i=1:size(mimplied,1)
    mimplied(i) = impliedmatches(AIMPLIED(i), S(i), v(i), supportA, SVECTOR, FAS1, v, supportsize,m,S,hh);
end

% COMPUTE ELASTICITY WRT VACANCIES
% Define variable that enters matching function (matching efficiency times
% observed search)
simplied = A.*S;
mJolts = mJolts(~(NaNindex)==1);

% Vector input into lasso
x_lasso = [simplied v simplied.*v simplied.^2 v.^2 (simplied.^2).*(v.^2) simplied.^3 v.^3 simplied.*v.^2 (simplied.^2).*v (simplied.^3).*(v.^3) (simplied.^3).*v (simplied.^3).*(v.^2) simplied.*(v.^3) (simplied.^2).*(v.^3)];
% derivative wrt to v
x_lasso_derv = [zeros(size(v,1),1) ones(size(v,1),1) simplied zeros(size(v,1),1) 2*v  2*(v).*(simplied.^2) zeros(size(v,1),1) 3*v.^2 2*(v).*simplied simplied.^2 3*(v.^2).*(simplied.^3) (simplied.^3) 2*v.*(simplied.^3) 3*(v.^2).*simplied 3*(v.^2).*(simplied.^2)];
% derivative wrt to S
x_lasso_derS = [ones(size(v,1),1) zeros(size(v,1),1) v 2*simplied zeros(size(v,1),1) 2*(simplied).*(v.^2) 3*simplied.^2 zeros(size(v,1),1) v.^2 2*(simplied).*v 3*(simplied.^2).*(v.^3) 3*(simplied.^2).*v 3*(simplied.^2).*(v.^2) (v.^3) 2*simplied.*(v.^3)];
%
% Perform lasso
[B,FitInfo] = lasso(x_lasso,m);
%
%
% figure(2311)
% plot(FitInfo.MSE)
%
% value of lambda, lasso parameter. here picked the point where MSE starts to climb.
% uncomment above figure to plot MSE for different values of lambda.
lam = 56;

% Estimated coefficients on x_lasso vector estimated
beta_lasso = B(:,lam);

% Compute elasticities elasticity wrt to v
elasticity_lassov = (x_lasso_derv*beta_lasso).*(v./m);

% elasticity wrt to S
elasticity_lassoS = (x_lasso_derS*beta_lasso).*(simplied./m);
%
sumelasticities2 = elasticity_lassoS+elasticity_lassov;

elasticity_lassov3 = elasticity_lassov./sumelasticities2;
elasticity_lassoS3 = elasticity_lassoS./sumelasticities2;


Fig_main2 = [mimplied m simplied elasticity_lassov3 A year month];
csvwrite('figmain_2.csv',Fig_main2);


% HOW MUCH VARIATION IN OBSERVED SUPPLY, DEMAND AND MATCHING EFFICIENCY
% CONTRIBUTED TO CHANGE IN HIRES IN GREAT RECESSION


% set observation where everything will remain fixed. using Oct 2007
% remember that month = (month-1)/12; and yearmonth = year+month;
month = (month-1)/12;
yearmonth = year+month;

[~, basedate] = min(abs(yearmonth-2007.75));
%
mimpliedAt = zeros(size(m,1),1);
for i=1:size(mimplied,1)
        mimplied(i) = impliedmatches(AIMPLIED(i), S(i), v(i), supportA, SVECTOR, FAS1, v, supportsize,m,S,hh);
    mimpliedAt(i) = impliedmatches(AIMPLIED(i), S(basedate), v(basedate), supportA,SVECTOR, FAS1, v, supportsize,m,S,hh);
end
mimpliedS = zeros(size(m,1),1);
for i=1:size(mimplied,1)
    mimpliedS(i) = impliedmatches(AIMPLIED(basedate), S(i), v(basedate), supportA,SVECTOR, FAS1, v, supportsize,m,S,hh);
end
mimpliedvt = zeros(size(m,1),1);
for i=1:size(mimplied,1)
    mimpliedvt(i) = impliedmatches(AIMPLIED(basedate), S(basedate), v(i), supportA,SVECTOR, FAS1, v, supportsize,m,S,hh);
end
%
mimpliedSvt = zeros(size(m,1),1);
for i=1:size(mimplied,1)
    mimpliedSvt(i) = impliedmatches(AIMPLIED(basedate), S(i), v(i), supportA,SVECTOR, FAS1, v, supportsize,m,S,hh);
end


% GRAPHS
%
figure(1)
scatter(yearmonth,vJolts)
set(gca,'fontsize',18)
xlabel('Year')
ylabel('Vacancies (JOLTS)')
xlim([min(year) max(yearmonth)])
%
figure(111)
scatter(yearmonth,dfhindex)
set(gca,'fontsize',18)
xlabel('Year')
ylabel('Recruiting Index')
xlim([min(year) max(yearmonth)])
%
figure(2)
scatter(yearmonth,S)
set(gca,'fontsize',18)
xlabel('Year')
ylabel('Observed Search, S_t')
xlim([min(year) max(yearmonth)])
%
hold on
smoothS=smooth(S,0.5,'loess');
scatter(yearmonth,smoothS,'m');
%
%
uniqueyear= unique(year);
Sannual = zeros(size(uniqueyear,1),1);
vannual = zeros(size(uniqueyear,1),1);
for i=1:size(uniqueyear)
    tempS=S(year==uniqueyear(i));
    Sannual(i) = mean(tempS);
    tempv=v(year==uniqueyear(i));
    vannual(i) = mean(tempv);
end
temp2 = num2str(uniqueyear);
labels2 = cellstr(temp2);

figure(3)
scatter(Sannual(1:7),vannual(1:7))
hold on
plot(Sannual(8:end),vannual(8:end))
set(gca,'fontsize',18)
title('Augmented Beveridge Curve')
xlabel('Aggregate Search')
ylabel('Vacancies (adj by recruiting effort)')
text(Sannual,vannual,labels2);
%    
%
figure(4)
scatter(m,mimplied)
set(gca,'fontsize',18)
xlabel('Actual Matches')
ylabel('Predicted Matches')
%
%
T=ones(size(v,1),1);
bA = regress(A,[T S]);
bv = regress(v,[T S]);
Aresid = A-bA(1).*T-bA(2).*S;
vresid = v-bv(1).*T-bv(2).*S;
%
figure(5)
scatter(vresid, Aresid)
set(gca,'fontsize',18)
xlabel('Recruiting Effort (residual)')
ylabel('Matching Efficiency (residual)')
%
figure(6)
scatter(yearmonth,elasticity_lassov3)
set(gca,'fontsize',18)
xlabel('Year')
ylabel('Elasticity wrt Vacancies')
xlim([min(year) max(yearmonth)])
%
hold on
smoothedelasticity3=smooth(elasticity_lassov3,0.5,'loess');
scatter(yearmonth,smoothedelasticity3,'m')
%
%
tightness=v./S;
figure(66)
scatter(tightness, elasticity_lassov3)
set(gca,'fontsize',18)
xlabel('Market Tightness, V_t/S_t')
ylabel('Elasticity wrt Vacancies')
ylim([min(elasticity_lassov3),max(elasticity_lassov3)])
%
figure(7)
scatter(yearmonth,A);
set(gca,'fontsize',18)
xlabel('Year')
ylabel('Matching Efficiency, A_t')
xlim([min(year) max(yearmonth)])
%
hold on
smoothA=smooth(A,0.5,'loess');
scatter(yearmonth,smoothA,'m')
%
month2 = (month2-1)/12;
yearmonth2 = year2+month2;
yearmonth2 = yearmonth2(~(isnan(s_u_new)==1));
s_u_new = s_u_new(~(isnan(s_u_new)==1));
figure(9)
scatter(yearmonth2,s_u_new)
set(gca,'fontsize',18)
xlabel('Year')
ylabel('Unemployed Search Effort')
xlim([min(year) max(yearmonth)])
%
hold on
s_u_original = s_u_original(~(isnan(s_u_original)==1));
s_u_original = [s_u_original;zeros(size(s_u_new,1)-size(s_u_original,1),1)];
scatter(yearmonth2,s_u_original)
ylim([25 40])
legend('Updated Measure','Original Measure')
%
% create search measure for employed workers and OLF (adding up search by
% women and men)
s_n = s_nm.*(Nmale./(Nmale+Nfemale))+ s_nf.*(Nfemale./(Nmale+Nfemale));
s_e = s_em.*(Emale./(Emale+Efemale))+ s_ef.*(Efemale./(Emale+Efemale));
%
figure(10)
scatter(yearmonth,s_e)
set(gca,'fontsize',18)
xlabel('Year')
ylabel('Employed Search Effort')
xlim([min(year) max(yearmonth)])
%
hold on
smooths_e=smooth(s_e,0.5,'loess');
scatter(yearmonth,smooths_e,'m')
%
figure(11)
scatter(yearmonth,s_n)
set(gca,'fontsize',18)
xlabel('Year')
ylabel('OLF Search Effort')
xlim([min(year) max(yearmonth)])
%
hold on
smooths_n=smooth(s_n,0.5,'loess');
scatter(yearmonth,smooths_n,'m')
%
figure(12)
scatter(tightness,A)
set(gca,'fontsize',18)
xlabel('Market Tightness, V_t/S_t')
ylabel('Matching Efficiency, A_t')
%
msmooth = smooth(mimplied,0.5,'loess');
mAtsmooth = smooth(mimpliedAt,0.5,'loess');
mSsmooth = smooth(mimpliedS,0.5,'loess');
mvtsmooth = smooth(mimpliedvt,0.5,'loess');
mSvtsmooth = smooth(mimpliedSvt,0.5,'loess');
%
figure(8)
plot(yearmonth,msmooth,'o',yearmonth,mAtsmooth,'+',yearmonth,mSsmooth,'*',yearmonth,mvtsmooth,'-*',yearmonth,mSvtsmooth);
set(gca,'fontsize',24)
xlabel('Year')
ylabel('Matches')
xlim([min(year) max(yearmonth)])
legend({'Actual', 'Only Matching Efficacy (A_t)','Only Observed Search (S_t)','Only Vacancies+Recruiting Effort','Observed Measures (S_t and Vacancy+Recruiting Effort)' }, 'Location', 'Best','TextColor','black')


% Read out data for figure 8
s_u_new1 = s_u_new(~(NaNindex)==1);
s_u_original1 = s_u_original(~(NaNindex)==1);
Fig_main3 = [msmooth mAtsmooth mSsmooth mvtsmooth mSvtsmooth s_u_new1 s_u_original1 s_e s_n A tightness year month];
csvwrite('figmain_3.csv',Fig_main3);


function mimplied = impliedmatches(A5, S5, v5, supportA, SVECTOR, FAS1, v, supportsize,m,S,hh)
%
% Fuction that backs out m
%
% for a given triplet (A5, S5, v5) back out the predicted m5
%
[~, pointA] = min(abs(supportA-A5));
[~, pointS] = min(abs(SVECTOR-S5));
Fas = FAS1(pointA,pointS); % find closest point in FAS1
%
% Calculate Gm|s,v for a given s and v
%
% create mvector
mvector = linspace(prctile(m,0.06),prctile(m,99.9),supportsize)';
%
hsvv = hh*range(v(:));
hsu = hh*max([range(S(:)) 1]);
%
% compute Gm|s,v at particular values S5 and v5

Gmsv = zeros(size(mvector,1),1);
for i=1:size(mvector,1)
    y = 1*(m(:)<=mvector(i));
    Gmsv(i) = sum(y.*mvnpdf([((v(:)-v5)/hsvv) ((S(:)-S5)/hsu)]))/sum(mvnpdf([((v(:)-v5)/hsvv) ((S(:)-S5)/hsu)]));
end

% find point in Gmsv that corresponds to Fas
[~, pointm] = min(abs(Gmsv-Fas));
mimplied = mvector(pointm);

end